<?php

defined( 'ABSPATH' ) or die( 'Access denied' );
if ( !class_exists( 'A_W_F' ) ) {
    class A_W_F
    {
        /** Current running instance of A_W_F object
         *
         * @since 1.0.0
         * @var A_W_F object
         */
        private static  $instance ;
        /** Current running instance of annasta admin
         *
         * @since 1.0.0
         * @var A_W_F_admin object
         */
        public static  $admin = null ;
        /** Current running instance of annasta frontend
         *
         * @since 1.0.0
         * @var A_W_F_frontend object
         */
        public static  $front = null ;
        /** Wc products taxonomy names that aren't allowed to become a filter taxonomy
         *
         * @since 1.0.0
         * @var array
         */
        public static  $excluded_taxonomies ;
        /** Master array of presets
         *
         * @since 1.0.0
         * @var array
         */
        public static  $presets = array() ;
        /** An array of ids for all the available modules
         *
         * @since 1.0.0
         * @var array
         */
        public static  $modules ;
        /** Shortcodes id count for the current session
         *
         * @since 1.0.0
         * @var array
         */
        private static  $sc_id = 1 ;
        /** Caching avoidance workaround
         *
         * @since 1.0.0
         * @var string
         */
        public static  $plugin_version ;
        private function __construct()
        {
            self::$plugin_version = A_W_F_VERSION;
            //self::$plugin_version = time();
            
            if ( !class_exists( 'WooCommerce' ) ) {
                if ( is_admin() ) {
                    add_action( 'load-plugins.php', array( $this, 'add_wc_absence_warning' ) );
                }
                return;
            }
            
            spl_autoload_register( array( $this, 'awf_autoloader' ) );
            add_action( 'init', array( $this, 'load_textdomain' ) );
            add_action( 'widgets_init', array( $this, 'register_widget' ) );
            add_action( 'init', array( $this, 'register_shortcode' ) );
            self::$excluded_taxonomies = array( 'product_type', 'product_visibility', 'product_shipping_class' );
            self::$presets = get_option( 'awf_presets', array() );
            if ( !is_array( self::$presets ) ) {
                self::$presets = array();
            }
            self::$modules = array(
                'taxonomy',
                'search',
                'price',
                'stock',
                'featured',
                'rating',
                'onsale',
                'ppp',
                'orderby'
            );
            // @ToDo 'meta'
            
            if ( is_admin() ) {
                
                if ( wp_doing_ajax() && isset( $_REQUEST['awf_front'] ) ) {
                    add_action( 'wp_ajax_awf', array( $this, 'frontend_ajax_controller' ) );
                    add_action( 'wp_ajax_nopriv_awf', array( $this, 'frontend_ajax_controller' ) );
                } else {
                    self::$admin = A_W_F_admin::get_instance();
                }
            
            } else {
                self::$front = A_W_F_frontend::get_instance();
                self::$front->filters_manager = 'A_W_F_filter_frontend';
            }
        
        }
        
        public static function get_instance()
        {
            if ( is_null( self::$instance ) ) {
                self::$instance = new self();
            }
            return self::$instance;
        }
        
        public static function activate_plugin()
        {
            if ( !current_user_can( 'activate_plugins' ) ) {
                return;
            }
            $plugin = ( isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '' );
            check_admin_referer( "activate-plugin_{$plugin}" );
            
            if ( empty(get_option( 'awf_presets' )) ) {
                $presets = array(
                    1 => array(
                    'position'     => 1,
                    'filters'      => array(),
                    'associations' => array(
                    0 => 'shop-pages',
                ),
                    'overrides'    => array(),
                ),
                );
                update_option( 'awf_presets', $presets );
                update_option( 'awf_preset_1_name', __( 'Default Preset', 'annasta-filters' ) );
            }
        
        }
        
        public static function deactivate_plugin()
        {
            if ( !current_user_can( 'activate_plugins' ) ) {
                return;
            }
            require 'class-a-w-f-admin.php';
            self::$admin = A_W_F_admin::get_instance();
            self::$admin->clear_product_counts_cache();
        }
        
        public static function uninstall_plugin()
        {
            if ( !current_user_can( 'activate_plugins' ) ) {
                return;
            }
            
            if ( empty(self::$admin) ) {
                require 'class-a-w-f-admin.php';
                self::$admin = A_W_F_admin::get_instance();
            }
            
            self::$admin->clear_product_counts_cache();
            unregister_widget( 'A_W_F_widget' );
            $all_options = wp_load_alloptions();
            $awf_options = array(
                'awf_version',
                'awf_premium',
                'awf_seo_filters_settings',
                'awf_page_title',
                'awf_shop_title',
                'awf_default_page_title',
                'awf_default_shop_title',
                'awf_add_seo_meta_description',
                'awf_seo_meta_description',
                'awf_seo_filters_title_prefix',
                'awf_seo_filters_title_postfix',
                'awf_seo_filters_separator',
                'awf_seo_filter_values_separator',
                'awf_shortcode_pages',
                'awf_shop_title_badges',
                'awf_remove_wc_orderby',
                'awf_ajax_scroll_on',
                'awf_ajax_scroll_adjustment',
                'awf_force_products_display_on',
                'awf_shop_columns',
                'awf_redirect_search',
                'awf_products_html_wrapper',
                'awf_pretty_scrollbars',
                'awf_color_image_style',
                'awf_user_css',
                'awf_ppp_default',
                'awf_include_children_on',
                'awf_cat_views_on',
                'awf_presets',
                'awf_custom_style',
                'awf_range_slider_style',
                'awf_product_cat_pretty_name',
                'awf_product_tag_pretty_name'
            );
            foreach ( $all_options as $name => $value ) {
                if ( 0 !== strpos( $name, 'awf_' ) ) {
                    continue;
                }
                if ( stristr( $name, 'awf_preset_' ) || stristr( $name, 'awf_filter_' ) || substr( $name, -12 ) === '_pretty_name' || in_array( $name, $awf_options ) ) {
                    delete_option( $name );
                }
            }
        }
        
        public function add_wc_absence_warning()
        {
            add_action( 'admin_notices', array( $this, 'display_wc_absence_warning' ) );
        }
        
        public function display_wc_absence_warning()
        {
            echo  '<div class="notice notice-warning is-dismissible"><p>', esc_html__( 'annasta Woocommerce Product Filters requires WooCommerce plugin to function properly. Please install WooCommerce online shop to use the annasta Woocommerce Product Filters plugin.', 'annasta-filters' ), '</p></div>' ;
        }
        
        public function load_textdomain()
        {
            load_plugin_textdomain( 'annasta-filters', false, plugin_basename( dirname( A_W_F_PLUGIN_FILE ) ) . '/languages' );
        }
        
        public function frontend_ajax_controller()
        {
            self::$front = A_W_F_frontend::get_instance();
            self::$front->ajax_controller();
        }
        
        public function register_widget()
        {
            register_widget( 'A_W_F_widget' );
        }
        
        function register_shortcode()
        {
            add_shortcode( 'annasta_filters', array( $this, 'filters_shortcode' ) );
        }
        
        public function filters_shortcode( $atts )
        {
            
            if ( !empty(self::$front) && !empty(self::$presets) ) {
                $sc_id = 'shortcode-' . self::$sc_id++;
                $filtered_atts = shortcode_atts( array(
                    'shortcode_id' => $sc_id,
                    'preset_id'    => '',
                    'preset_name'  => '',
                ), $atts );
                if ( !empty($filtered_atts['preset_name']) ) {
                    foreach ( self::$presets as $id => $data ) {
                        $name = get_option( 'awf_preset_' . $id . '_name', '' );
                        
                        if ( $name === $filtered_atts['preset_name'] ) {
                            $filtered_atts['preset_id'] = $id;
                            break;
                        }
                    
                    }
                }
                $filtered_atts['preset_id'] = (int) $filtered_atts['preset_id'];
                if ( isset( self::$presets[$filtered_atts['preset_id']] ) ) {
                    return self::$front->display_shortcode( $filtered_atts );
                }
            }
            
            return '';
        }
        
        public static function get_seo_meta_description( $query )
        {
            $description = stripcslashes( get_option( 'awf_seo_meta_description', '' ) );
            $parts = self::get_seo_parts( $query );
            $filters = '';
            if ( isset( $parts['empty_filters'] ) ) {
                $filters .= implode( ' ', $parts['empty_filters'] ) . ' ';
            }
            $filters .= implode( get_option( 'awf_seo_filters_separator', ' - ' ), $parts['filters'] ) . ' ';
            if ( isset( $parts['disabled'] ) ) {
                $filters .= implode( get_option( 'awf_seo_filters_separator', ' - ' ), $parts['disabled'] );
            }
            $filters = str_replace( " ", ' ', $filters );
            $filters = str_replace( '  ', ' ', trim( $filters ) );
            $description = str_replace( '{annasta_filters}', $filters, $description );
            return $description;
        }
        
        public static function get_seo_title( $query, $type = 'page' )
        {
            $parts = self::get_seo_parts( $query );
            $title = '';
            
            if ( empty($parts['filters']) ) {
                $title = get_option( 'awf_default_' . $type . '_title', _x( 'Shop', 'Default page title', 'annasta-filters' ) );
            } else {
                $title = get_option( 'awf_seo_filters_title_prefix', __( 'Shop for ', 'annasta-filters' ) ) . ' ';
                if ( isset( $parts['empty_filters'] ) ) {
                    $title .= implode( ' ', $parts['empty_filters'] ) . ' ';
                }
                $title .= implode( get_option( 'awf_seo_filters_separator', ' - ' ), $parts['filters'] ) . ' ';
                $title .= get_option( 'awf_seo_filters_title_postfix', '' );
            }
            
            $title = str_replace( " ", ' ', $title );
            $title = str_replace( '  ', ' ', trim( $title ) );
            return $title;
        }
        
        private static function get_seo_parts( $query )
        {
            $get_all = ( 'yes' === get_option( 'awf_add_seo_meta_description', 'no' ) ? true : false );
            $seo_parts = array(
                'filters' => array(),
            );
            if ( !empty(self::$front) ) {
                
                if ( empty(self::$front->seo_parts) ) {
                    self::$front->seo_parts = array(
                        'filters' => array(),
                    );
                    $seo_parts =& self::$front->seo_parts;
                } else {
                    return self::$front->seo_parts;
                }
            
            }
            $filters_settings = get_option( 'awf_seo_filters_settings', array() );
            foreach ( $filters_settings as $filter => $settings ) {
                $part_name = 'filters';
                if ( empty($settings['enabled']) ) {
                    
                    if ( $get_all ) {
                        $part_name = 'disabled';
                    } else {
                        continue;
                    }
                
                }
                
                if ( 0 === strpos( $filter, 'taxonomy_' ) ) {
                    $taxonomy = substr( $filter, strlen( 'taxonomy_' ) );
                    
                    if ( !empty($query->tax[$taxonomy]) ) {
                        $names = array();
                        foreach ( $query->tax[$taxonomy] as $slug ) {
                            $term = get_term_by( 'slug', $slug, $taxonomy );
                            if ( !empty($term) ) {
                                $names[] = $term->name;
                            }
                        }
                        $seo_parts[$part_name][] = $settings['prefix'] . implode( get_option( 'awf_seo_filter_values_separator', ', ' ), $names ) . $settings['postfix'];
                    } elseif ( !empty($settings['empty']) ) {
                        $seo_parts['empty_filters'][] = $settings['empty'];
                    }
                
                } else {
                    
                    if ( !empty($query->awf[$filter]) ) {
                        
                        if ( 'search' === $filter ) {
                            $seo_parts[$part_name][] = $settings['prefix'] . $query->awf[$filter] . $settings['postfix'];
                        } elseif ( isset( $settings['labels'] ) && isset( $settings['labels'][$query->awf[$filter]] ) ) {
                            $seo_parts[$part_name][] = $settings['prefix'] . $settings['labels'][$query->awf[$filter]] . $settings['postfix'];
                        }
                    
                    } elseif ( isset( $query->awf['min_' . $filter] ) || isset( $query->awf['max_' . $filter] ) ) {
                        $min = ( isset( $query->awf['min_' . $filter] ) ? $query->awf['min_' . $filter] : $settings['range_min'] );
                        $max = ( isset( $query->awf['max_' . $filter] ) ? $query->awf['max_' . $filter] : $settings['range_max'] );
                        
                        if ( 'price' === $filter ) {
                            $decimals = wc_get_price_decimals();
                            $decimal_separator = wc_get_price_decimal_separator();
                            $thou_separator = wc_get_price_thousand_separator();
                            $min = number_format(
                                $min,
                                $decimals,
                                $decimal_separator,
                                $thou_separator
                            );
                            $max = number_format(
                                $max,
                                $decimals,
                                $decimal_separator,
                                $thou_separator
                            );
                            $symbol = get_woocommerce_currency_symbol( get_woocommerce_currency() );
                            switch ( get_option( 'woocommerce_currency_pos', '' ) ) {
                                case 'left':
                                    $min = $symbol . $min;
                                    $max = $symbol . $max;
                                    break;
                                case 'right':
                                    $min .= $symbol;
                                    $max .= $symbol;
                                    break;
                                case 'left_space':
                                    $min = $symbol . ' ' . $min;
                                    $max = $symbol . ' ' . $max;
                                    break;
                                case 'right_space':
                                    $min .= ' ' . $symbol;
                                    $max .= ' ' . $symbol;
                                    break;
                            }
                        }
                        
                        $seo_parts[$part_name][] = $settings['prefix'] . $min . $settings['range_separator'] . $max . $settings['postfix'];
                    } else {
                        if ( !empty($settings['empty']) ) {
                            $seo_parts['empty_filters'][] = $settings['empty'];
                        }
                    }
                
                }
            
            }
            return apply_filters( 'awf_seo_filters', $seo_parts );
        }
        
        private function awf_autoloader( $class )
        {
            if ( 0 !== strpos( $class, 'A_W_F_' ) ) {
                return;
            }
            $class = strtolower( $class );
            $path = A_W_F_PLUGIN_PATH . 'code/';
            $file = 'class-' . str_replace( '_', '-', $class ) . '.php';
            $file_path = $path . $file;
            if ( is_readable( $file_path ) ) {
                include $file_path;
            }
        }
        
        public static function enqueue_style_options_css()
        {
            $uploads_dir = wp_upload_dir();
            
            if ( file_exists( $uploads_dir['basedir'] . '/annasta-filters/style-options.css' ) ) {
                $style_options_url = $uploads_dir['baseurl'] . '/annasta-filters/style-options.css';
                if ( is_ssl() ) {
                    $style_options_url = str_replace( 'http://', 'https://', $style_options_url );
                }
                wp_enqueue_style(
                    'awf-style-options',
                    $style_options_url,
                    array(),
                    A_W_F::$plugin_version
                );
            }
        
        }
        
        private final function __clone()
        {
        }
        
        // prevent cloning
        private final function __wakeup()
        {
        }
        
        // prevent serialization
        /** Helper Functions */
        public static function format_print_r( $print )
        {
            print "<pre>";
            print_r( $print );
            print "</pre>";
            //A_W_F::format_print_r( self::$features_by_parent[0] );
        }
    
    }
}